__doc__ = """CachePool module"""
__version__ = '0.2.1'

from Globals import HTMLFile
from Globals import MessageDialog
from Globals import Persistent

import OFS.SimpleItem
import Acquisition
import AccessControl.Role
import urllib
import time
from string import split, join

manage_addCachePoolForm = HTMLFile('CachePoolAdd', globals())

def manage_addCachePool(self,id,title,REQUEST=None):
  """Add CachePool Instance"""
  self._setObject(id,CachePool(id,title))
  if REQUEST is not None:
    return self.manage_main(self, REQUEST)

class CachePool(
  Persistent,
  Acquisition.Implicit,
  AccessControl.Role.RoleManager,
  OFS.SimpleItem.Item,
  ):
  "Cache Pool product"

  index_html=None # Prevent accidental acquisition
  meta_type = 'Cache Pool'

  meta_types = ({'permission': 'Add CachePool', 'name': 'CachePool', 'product':
'CachePool', 'action': 'manage_addProduct/CachePool/CachePoolAdd'})

  manage_options=(
      {'label':'Properties','action':'manage_editForm'},
      {'label':'View Cache','action':'manage_viewCache'},
      {'label':'Security','action':'manage_access'},
  )

  __ac_permissions__=(
      ('View management screens',('manage_tabs','manage_main')),
      ('Edit Properties',('manage_editForm','manage_edit')),
      ('Change permissions',('manage_access',)),
  )

  def __init__(self,id,title):
    self.id = id
    self.title = title
    self.TTL = 60
    self.DocToParse = 'index_orig'
    self.CacheSlots = 5
    self.Cache = []

  def __call__(self,REQUEST=None,RESPONSE=None,**kw):
    """Process cache slots"""
    kw['document_id']   =self.id
    kw['document_title']=self.title
    pos = self.findInCache()

    #If there's no more place...
    if pos == -1:
      return urllib.urlopen(self.REQUEST['URL1']+'/'+self.DocToParse+'?'+self.REQUEST['QUERY_STRING']).read()

    self.Cache[pos].hpcc = self.Cache[pos].hpcc  + 1

    if ((int(self.Cache[pos].CacheDate) + int(self.TTL)) <= time.time()):
      """Invalidate CacheSlot"""
      self.Cache[pos].Cached = None

    if self.Cache[pos].Cached == None:
      """Refresh CacheSlot"""
      self.Cache[pos].Last_hpcc = self.Cache[pos].hpcc
      self.Cache[pos].hpcc = 1
      self.Cache[pos].QueryString = self.REQUEST['QUERY_STRING']
      self.Cache[pos].URL1 = self.REQUEST['URL1']
      self.Cache[pos].CacheData = urllib.urlopen(self.REQUEST['URL1']+'/'+self.DocToParse+'?'+self.REQUEST['QUERY_STRING']).read()
      self.Cache[pos].CacheDate = int(time.time())
      self.Cache[pos].Cached = 1
    else:
      pass
    return self.Cache[pos].CacheData

  def findInCache(self):
    """Search the cacheslot pool for an instance with the same URL1 and QueryString.
        If none is found and there are free slots, add a fresh one, else return -1"""
    for pos in range(len(self.Cache)):
      # Try to find one with the same properties
      if (self.Cache[pos].URL1 == self.REQUEST['URL1']) and (self.Cache[pos].QueryString == self.REQUEST['QUERY_STRING']):
	return pos
    if len(self.Cache) < int(self.CacheSlots):
      # Add a slot if space is available
      self.Cache.append(CacheSlot())
      return len(self.Cache) - 1
    # Find expired with the lowest hppc, bigger than 0(coz these are new, uncycled)
    Expired = []
    for pos in range(len(self.Cache)):
      if (self.Cache[pos].hpcc < 1) or ((int(self.Cache[pos].CacheDate) + int(self.TTL)) > time.time()):
        Expired.append(9999999)
      else:
        Expired.append(self.Cache[pos].hpcc)
    if min(Expired) < 9999999:
      del self.Cache[(Expired.index(min(Expired)))]
      self.Cache.append(CacheSlot())
      return len(self.Cache) - 1
    return -1


  def now(self):
    """Now"""
    return time.time()

  def flush_cache(self,Slot=None,REQUEST=None):
    """Flush some cache slots"""
    if Slot == None:
      self.Cache = []
    else:
      try:
        Slot = split(Slot)
      except:
        pass
      Slot.reverse()
      for item in Slot:
        del self.Cache[int(item)]
    if REQUEST is not None:
        return MessageDialog(
            title='Flush Cache',
            message='<strong>%s</strong>\'s caches have been flushed.' %
            self.id,
            action ='./manage_viewCache',
              )

  manage_viewCache = HTMLFile('CachePoolView',globals())

  ######################################################################
  #EDIT STUFF
  manage_editForm = HTMLFile('CachePoolEdit',globals())
  def manage_edit(self,title,TTL,DocToParse,CacheSlots,REQUEST=None):
      "edit Cache Pool characteristics"
      self.title=title
      self.TTL = TTL
      self.DocToParse = DocToParse
      if CacheSlots < self.CacheSlots:
        self.Cache = self.Cache[:int(CacheSlots)]
      self.CacheSlots = CacheSlots
      if REQUEST is not None:
          return MessageDialog(
              title='Edited',
              message='<strong>%s</strong> has been edited.' %
              self.id,
              action ='./manage_editForm',
                )



class CacheSlot:
  """Cache Slot Definition"""

  def __init__(self):
    self.URL1 = ''
    self.QueryString = ''
    self.hpcc = 0
    self.Last_hppc = -1
    self.CacheData = ''
    self.CacheDate = 0
    self.Cached = None
